package com.totopia.server.config.security;

import com.fasterxml.jackson.annotation.JsonIgnore;
import com.totopia.server.modules.user.entity.UserEntity;
import org.springframework.security.core.GrantedAuthority;
import org.springframework.security.core.authority.SimpleGrantedAuthority;
import org.springframework.security.core.userdetails.UserDetails;

import java.util.Collection;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;

public class SecurityUserDetails implements UserDetails {
  private static final long serialVersionUID = 1L;

  private Long id;

  private String name;

  private String username;

  @JsonIgnore
  private String email;

  @JsonIgnore
  private String password;

  private Collection<? extends GrantedAuthority> authorities;

    public SecurityUserDetails(Long id, String name, String username, String email, String password, Collection<? extends GrantedAuthority> authorities) {
        this.id = id;
        this.name = name;
        this.username = username;
        this.email = email;
        this.password = password;
        this.authorities = authorities;
    }

    public SecurityUserDetails() {
    }

    public static SecurityUserDetailsBuilder builder() {
        return new SecurityUserDetailsBuilder();
    }

  @Override
  public String getUsername() {
    return username;
  }

  @Override
  public String getPassword() {
    return password;
  }

  @Override
  public Collection<? extends GrantedAuthority> getAuthorities() {
    return authorities;
  }

  @Override
  public boolean isAccountNonExpired() {
    return true;
  }

  @Override
  public boolean isAccountNonLocked() {
    return true;
  }

  @Override
  public boolean isCredentialsNonExpired() {
    return true;
  }

  @Override
  public boolean isEnabled() {
    return true;
  }

  @Override
  public boolean equals(Object o) {
    if (this == o)
      return true;
    if (o == null || getClass() != o.getClass())
      return false;
    SecurityUserDetails that = (SecurityUserDetails) o;
    return Objects.equals(id, that.id);
  }

  @Override
  public int hashCode() {
    return Objects.hash(id);
  }

  public static SecurityUserDetails create(UserEntity user) {
    List<GrantedAuthority> authorities = user.getRoles().stream()
        .map(role -> new SimpleGrantedAuthority(role.getName().name())).collect(Collectors.toList());

    return SecurityUserDetails.builder().username(user.getUsername()).password(user.getPassword())
        .authorities(authorities).build();
  }

    public Long getId() {
        return this.id;
    }

    public String getName() {
        return this.name;
    }

    public String getEmail() {
        return this.email;
    }

    public void setId(Long id) {
        this.id = id;
    }

    public void setName(String name) {
        this.name = name;
    }

    public void setUsername(String username) {
        this.username = username;
    }

    public void setEmail(String email) {
        this.email = email;
    }

    public void setPassword(String password) {
        this.password = password;
    }

    public void setAuthorities(Collection<? extends GrantedAuthority> authorities) {
        this.authorities = authorities;
    }

    public String toString() {
        return "SecurityUserDetails(id=" + this.getId() + ", name=" + this.getName() + ", username=" + this.getUsername() + ", email=" + this.getEmail() + ", password=" + this.getPassword() + ", authorities=" + this.getAuthorities() + ")";
    }

    public static class SecurityUserDetailsBuilder {
        private Long id;
        private String name;
        private String username;
        private String email;
        private String password;
        private Collection<? extends GrantedAuthority> authorities;

        SecurityUserDetailsBuilder() {
        }

        public SecurityUserDetails.SecurityUserDetailsBuilder id(Long id) {
            this.id = id;
            return this;
        }

        public SecurityUserDetails.SecurityUserDetailsBuilder name(String name) {
            this.name = name;
            return this;
        }

        public SecurityUserDetails.SecurityUserDetailsBuilder username(String username) {
            this.username = username;
            return this;
        }

        public SecurityUserDetails.SecurityUserDetailsBuilder email(String email) {
            this.email = email;
            return this;
        }

        public SecurityUserDetails.SecurityUserDetailsBuilder password(String password) {
            this.password = password;
            return this;
        }

        public SecurityUserDetails.SecurityUserDetailsBuilder authorities(Collection<? extends GrantedAuthority> authorities) {
            this.authorities = authorities;
            return this;
        }

        public SecurityUserDetails build() {
            return new SecurityUserDetails(id, name, username, email, password, authorities);
        }

        public String toString() {
            return "SecurityUserDetails.SecurityUserDetailsBuilder(id=" + this.id + ", name=" + this.name + ", username=" + this.username + ", email=" + this.email + ", password=" + this.password + ", authorities=" + this.authorities + ")";
        }
    }
}
