package com.totopia.server.modules.user.controller;

import com.totopia.server.commons.exception.ResourceNotFoundException;
import com.totopia.server.modules.user.entity.UserEntity;
import com.totopia.server.modules.user.repository.UserRepository;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.data.domain.Sort;
import org.springframework.data.web.PageableDefault;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

@RestController
public class UserController {
  @Autowired
  private UserRepository userRepository;

  @PostMapping(value = "/users")
  @ResponseStatus(code = HttpStatus.CREATED)
  public UserEntity save(@RequestBody UserEntity user) {
    return userRepository.save(user);
  }

  @GetMapping(value = "/users")
  public @ResponseBody Page<UserEntity> all(@PageableDefault(sort = {"username"}, direction = Sort.Direction.DESC, size = 10)Pageable pageable) {
    Page<UserEntity> users = userRepository.findAll(pageable);
//    Gson gson = new Gson();
//
//    String json = gson.toJson(users);
    return users;
//  public Page<UserEntity> all(Pageable pageable) {
//    return userRepository.findAll(pageable);
  }

  @GetMapping(value = "/users/{userId}")
  public UserEntity findByUserId(@PathVariable Long userId) {
    return userRepository.findById(userId).orElseThrow(() -> new ResourceNotFoundException("User", "userId", userId));
  }

  @DeleteMapping(value = "/users/{userId}")
  public ResponseEntity<?> deleteUser(@PathVariable Long userId) {

    return userRepository.findById(userId).map(user -> {
      userRepository.delete(user);
      return ResponseEntity.ok().build();
    }).orElseThrow(() -> new ResourceNotFoundException("User", "userId", userId));

  }

  @PutMapping(value = "/users/{userId}")
  public ResponseEntity<UserEntity> updateUser(@PathVariable Long userId, @RequestBody UserEntity newUser) {

    return userRepository.findById(userId).map(user -> {

      userRepository.save(user);
      return ResponseEntity.ok(user);
    }).orElseThrow(() -> new ResourceNotFoundException("User", "userId", userId));

  }
}
